#ifndef SENSOR_CLI_H_
#define SENSOR_CLI_H_

#include <zephyr/bluetooth/mesh.h>
#include <bluetooth/mesh/model_types.h>

/** Company ID of the Bluetooth Mesh Chat Client model. */
#define COMPANY_ID CONFIG_BT_COMPANY_ID

/** Model ID of the Bluetooth Mesh Chat Client model. */
#define SENSOR_MODEL_ID 0x000A

// Buffer for Message Send
#define BT_MESH_SENSOR_INFO_PUBLISH_OP_MESSAGE BT_MESH_MODEL_OP_3(0x10, COMPANY_ID)
#define MESSAGE_PUBLISH_LEN 100

// Buffer for Message Recieve
#define BT_MESH_SENSOR_INFO_RECIEVE_OP_MESSAGE BT_MESH_MODEL_OP_3(0x11, COMPANY_ID)

// Buffer for Acknolage
#define BT_MESH_SENSOR_INFO_ACK_OP_MESSAGE BT_MESH_MODEL_OP_3(0x12, COMPANY_ID)
#define MESSAGE_ACK_LEN 100

// Forward declaration
struct tranciever_client;

// Define Pub instance
BT_MESH_MODEL_PUB_DEFINE(_tranciever_model_pub, NULL, 4);

////////////////////////
// Define model instance
////////////////////////
#define BT_MESH_RX_TX_MODEL(_transceiver) BT_MESH_MODEL_VND_CB(COMPANY_ID, SENSOR_MODEL_ID, _transceiver_model_op, &(_transceiver)->pub, BT_MESH_MODEL_USER_DATA(struct tranciever_client, _transceiver), &tranciever_client_cb)

// Initialization implementations
int settings_set_inpl(struct bt_mesh_model *model, const char *name,
                      size_t len_rd, settings_read_cb read_cb,
                      void *cb_arg);

int start_inpl(struct bt_mesh_model *model);

int init_inpl(struct bt_mesh_model *model);

void reset_inpl(struct bt_mesh_model *model);

int tranciever_message_send(const struct tranciever_client *tranciever, uint16_t addr, const uint8_t *msg);

// Struct to define the tranciever specific functions
struct tranciever_handlers
{

    void (*start)(struct tranciever_client *chat);

    void (*tx_data)(uint16_t addr, uint8_t *msg);

    void (*rx_data)(struct tranciever_client *chat, uint8_t *msg);

    uint8_t *(*reply)(struct bt_mesh_msg_ctx *ctx, uint8_t *msg);
};

// Tranciever "Class" Object
struct tranciever_client
{
    struct bt_mesh_model *model;
    struct bt_mesh_model_pub pub;
    struct net_buf_simple pub_msg;
    uint8_t buf[BT_MESH_MODEL_BUF_LEN(
        BT_MESH_SENSOR_INFO_PUBLISH_OP_MESSAGE,
        MESSAGE_PUBLISH_LEN)];
    /** Handler function structure. */
    const struct tranciever_handlers *handlers;
};

extern const struct bt_mesh_model_op _transceiver_model_op[];
extern const struct bt_mesh_model_cb tranciever_client_cb;

#endif // SENSOR_CLI_H_
