#include "sensor_cli.h"
#include <string.h>

int settings_set_inpl(struct bt_mesh_model *model, const char *name,
                      size_t len_rd, settings_read_cb read_cb,
                      void *cb_arg)
{
    //struct tranciever_client *chat = model->user_data;

    //if (name)
    //{
    //    return -ENOENT;
    //}

    // Could be implemented if custom data should be stored
    /*
    ssize_t bytes = read_cb(cb_arg, &chat->presence, sizeof(chat->presence));
    if (bytes < 0) {
        return bytes;
    }
    if (bytes != 0 && bytes != sizeof(chat->presence)) {
        return -EINVAL;
    }
    */

    return 0;
}

int start_inpl(struct bt_mesh_model *model)
{
    struct tranciever_client *pTrance = model->user_data;

    // Call start function if its available
    if (pTrance->handlers->start)
        pTrance->handlers->start(pTrance);

    return 0;
}

struct bt_mesh_model_pub pub_ctx;

int init_inpl(struct bt_mesh_model *model)
{

    struct tranciever_client *pTranc = model->user_data;
    pTranc->model = model;
    pTranc->pub = _tranciever_model_pub;

    //pTranc->pub.msg = &pTranc->pub_msg;
    

    // Maybe implement sometimes
    // pTranc->pub.update = bt_mesh_chat_cli_update_handler;
    return 0;
}

void reset_inpl(struct bt_mesh_model *model)
{
    struct tranciever_client *pTrance = (struct tranciever_client *)model->user_data;

    // Store to persistent storage
    if (IS_ENABLED(CONFIG_BT_SETTINGS))
    {
        (void)bt_mesh_model_data_store(model, true, NULL, NULL, 0);
    }
}

int tranciever_message_send(const struct tranciever_client *tranciever, uint16_t addr, const uint8_t *msg)
{
    struct bt_mesh_msg_ctx ctx = {
        .addr = addr,
        .app_idx = tranciever->model->keys[0],
        .send_ttl = BT_MESH_TTL_DEFAULT,
        .send_rel = false,
    };

    // Define TX_Buffer
    BT_MESH_MODEL_BUF_DEFINE(buf, BT_MESH_SENSOR_INFO_PUBLISH_OP_MESSAGE, MESSAGE_PUBLISH_LEN);

    bt_mesh_model_msg_init(&buf, BT_MESH_SENSOR_INFO_PUBLISH_OP_MESSAGE);

    net_buf_simple_add_mem(&buf, msg, strnlen(msg, MESSAGE_PUBLISH_LEN));
    net_buf_simple_add_u8(&buf, '\0');

    return bt_mesh_model_send(tranciever->model, &ctx, &buf, NULL, NULL);
}

static const uint8_t *extract_msg(struct net_buf_simple *buf)
{
    buf->data[buf->len - 1] = '\0';
    return net_buf_simple_pull_mem(buf, buf->len);
}

static int handle_message_set(struct bt_mesh_model *model, struct bt_mesh_msg_ctx *ctx, struct net_buf_simple *buf)
{
    struct tranciever_client *pClient = model->user_data;
    uint8_t *msg;

    msg = extract_msg(buf);

    BT_MESH_MODEL_BUF_DEFINE(reply, BT_MESH_SENSOR_INFO_ACK_OP_MESSAGE, MESSAGE_ACK_LEN);
    bt_mesh_model_msg_init(&reply, BT_MESH_SENSOR_INFO_ACK_OP_MESSAGE);

    if (pClient->handlers->reply)
    {
        msg = pClient->handlers->reply(msg, ctx);
    }

    net_buf_simple_add_mem(&reply, &msg, strnlen(msg, (size_t)MESSAGE_PUBLISH_LEN));
    net_buf_simple_add_u8(&reply, '\0');

    return bt_mesh_model_send(model, ctx, &reply, NULL, NULL);
}

static int handle_message(struct bt_mesh_model *model, struct bt_mesh_msg_ctx *ctx, struct net_buf_simple *buf)
{
    struct tranciever_client *pClient = model->user_data;
    uint8_t *msg;

    msg = extract_msg(buf);

    if (pClient->handlers->rx_data)
    {
        pClient->handlers->rx_data(pClient, msg);
    }

    return 0;
}

const struct bt_mesh_model_op _transceiver_model_op[] = {
    {
        BT_MESH_SENSOR_INFO_RECIEVE_OP_MESSAGE,
        BT_MESH_LEN_MIN(MESSAGE_PUBLISH_LEN),
        handle_message
    },
    {
        BT_MESH_SENSOR_INFO_ACK_OP_MESSAGE,
        BT_MESH_LEN_MIN(MESSAGE_ACK_LEN),
        handle_message_set
    },
    BT_MESH_MODEL_OP_END,
};

const struct bt_mesh_model_cb tranciever_client_cb = {
    .init = init_inpl,
    .reset = reset_inpl,
    .settings_set = settings_set_inpl,
    .start = start_inpl,
};